// ShowTNC2 : Programme d'affichage des trames AX25 au format TNC2.
// Adapté par Frank LOBA (www.f4hdb.fr).
// Date création : 18/09/2024.
// Dernière mise à jour : 20/09/2024.


#include <sys/socket.h>
#include <netax25/ax25.h>
#include <netax25/axlib.h>
#include <netax25/axconfig.h>
#include <netinet/if_ether.h>
 
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "KissLib.h"

 
/**** DEFINITION DE LA FONCTION PRINCIPALE ****/ 
int main(int argc, char **argv)
{
  // Déclaration des variables locales.  
  int             SocketFd, Len;
  struct sockaddr SocketAdress;
  socklen_t       SocketAdressSize;
  char            BufferRecv[1000];
  char            BufferAX25[1000];
  char            BufferTNC2[1000];
  time_t          Maintenant;
  struct tm       *Local;

  // Détection de la présence de ports AX25 configurés.
  if(ax25_config_load_ports() == 0)
  {
    printf("no AX.25 port data configured\n");
    exit(1);
  }

  // Ouverture de la socket.
  SocketFd = socket(PF_PACKET, SOCK_PACKET, htons(ETH_P_AX25));

  if(SocketFd < 0)
  {
    printf("Error opening socket\n");
    exit(1);
  }
  
  // Gestion de la boucle principale.
  while(true)
  {
    // Affectation de la taille de l'adresse de la socket.
    SocketAdressSize = sizeof(SocketAdress);

    // R2ception de données brutes à travers la socket.
    if ((Len = recvfrom(SocketFd, BufferRecv, sizeof(BufferRecv), 0, &SocketAdress, &SocketAdressSize)) < 0)
    {
      printf("Error receiving data\n");
      exit(1);
    }

    // Affichage du port AX25.
    printf("%s ", ax25_config_get_name(SocketAdress.sa_data));

    // Affichage de l'heure et de la date.
    time(&Maintenant);
    Local = localtime(&Maintenant);

    printf("%02d:%02d:%02d %02d/%02d/%d ", Local->tm_hour, Local->tm_min, Local->tm_sec, Local->tm_mday, Local->tm_mon + 1, Local->tm_year + 1900);

    // Uniformisation des données reçues.
    UniformiserTrame(BufferRecv, BufferAX25, Len);

    // Conversion de la trame AX25 au format TNC2, puis affichage.
    if(AX25toTNC2(BufferAX25, BufferTNC2)) printf("%s\n", BufferTNC2);
    else printf("Error converting AX25 to TNC2\n");
  }

  // Fin de la fonction principale.
  return 0;  
}